import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class CustomInputText extends StatefulWidget {
  final String placeholder;
  final IconData? icon;
  final TextEditingController textController;
  final TextInputType keyboardType;
  final bool isPassword;
  final bool isAName;
  final bool isAPhone;
  final void Function(String)? onChanged;
  final bool? validation;
  final bool enabled;
  final Color backgroundColor;
  final Color onfocusColor;
  final bool shadow;
  final double borderRadius;

  const CustomInputText({
    required this.placeholder,
    required this.textController,
    this.icon,
    this.keyboardType = TextInputType.text,
    this.isPassword = false,
    this.isAName = false,
    this.isAPhone = false,
    this.onChanged,
    this.validation,
    this.enabled = true,
    this.backgroundColor = Colors.white,
    this.onfocusColor = Colors.blue,
    this.shadow = true,
    this.borderRadius = 30.0,
  });

  @override
  _CustomInputTextState createState() => _CustomInputTextState();
}

class _CustomInputTextState extends State<CustomInputText> {
  bool dontShow = true;
  bool emailValid = true;

  @override
  Widget build(BuildContext context) {
    return Container(
      //If the atribute isPassword is true, the padding right will be 0
      padding: (widget.isPassword)
          ? EdgeInsets.only(right: 0)
          // else if isPassword is false and the icon is null, the padding will be left: 15,right: 15
          // or if isPassword is false and the icon different from null, the padding only will be right: 15
          : (widget.icon == null)
              ? EdgeInsets.only(left: 15, right: 15)
              : EdgeInsets.only(right: 15),
      decoration: BoxDecoration(
          // The attribute backgroundColor is the background color that will have the textField
          color: widget.backgroundColor,
          border: (widget.keyboardType == TextInputType.emailAddress &&
                  widget.onChanged == null)
              //when the attribute keyboardType == TextInputType.emailAddress
              //by default the correct or incorrect email validation will be applied
              ? Border.all(
                  color: (emailValid) ? Colors.black26 : (Colors.red[600])!,
                )
              //This parameter can be used when the textfiel isn't keyboardType == TextInputType.emailAddress
              //you can change the color of borders when your validation is right or wrong
              : (widget.validation != null)
                  ? Border.all(
                      color: (widget.validation == true)
                          ? Colors.green
                          : Colors.red,
                    )
                  //if "validation is null" the color by default will be Colors.black26
                  : Border.all(color: Colors.black26),
          // the attribute widget.borderRadius will be the BorderRadius.circular
          // that the text field will have, defaul borderRadius is 30.0
          borderRadius: BorderRadius.circular(widget.borderRadius),
          boxShadow: <BoxShadow>[
            // The attribute shadow is true by default but it can be false
            // so that no shadows are applied to the text field
            if (widget.shadow)
              BoxShadow(
                  color: Colors.black.withOpacity(0.05),
                  offset: Offset(0, 5),
                  blurRadius: 5)
          ]),
      // the attribute onfocusColor will change the primary color of the textField
      child: Theme(
        data: Theme.of(context).copyWith(
          // override textfield's icon color when selected
          colorScheme: ThemeData().colorScheme.copyWith(
                primary: (widget.onfocusColor),
              ),
        ),
        //here the text field changes its sizes
        //and attributes to display a traditional password text field
        child: (widget.isPassword)
            ? Container(
                padding: EdgeInsets.only(right: 10),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Expanded(
                      child: Container(
                        child: TextField(
                          controller: widget.textController,
                          onChanged: widget.onChanged,
                          cursorColor: widget.onfocusColor,
                          autocorrect: false,
                          keyboardType: widget.keyboardType,
                          // if the attribute isPassword is true, the obscureText is true by deafult
                          // but this can change to false with the show password button
                          obscureText: dontShow,
                          decoration: InputDecoration(
                              prefixIcon: Icon(widget.icon),
                              focusedBorder: InputBorder.none,
                              border: InputBorder.none,
                              // the attribute placeholder is the hintText from the textField
                              // to show a info message about of the textField
                              hintText: widget.placeholder),
                        ),
                      ),
                    ),
                    // this is the show or hide password button
                    IconButton(
                        icon: Icon(
                          (dontShow)
                              ? Icons.visibility_off
                              : Icons.visibility_rounded,
                          color: Colors.black45,
                        ),
                        onPressed: () {
                          setState(() {
                            dontShow = !dontShow;
                          });
                        })
                  ],
                ),
              )
            : (widget.icon != null)
                ? TextField(
                    //the attribute enabled is used to define if the text field will be enabled or not
                    enabled: widget.enabled,
                    //The attribute textController is a TextInputType.text by default
                    controller: widget.textController,
                    // the validation to emails only workd when the attribute onChanged is null
                    onChanged:
                        (widget.keyboardType == TextInputType.emailAddress &&
                                widget.onChanged == null)
                            ? (value) {
                                final isValid = validateEmail(value);
                                setState(() {
                                  emailValid = isValid;
                                });
                              }
                            : widget.onChanged,
                    cursorColor: widget.onfocusColor,
                    textCapitalization: (widget.keyboardType ==
                            TextInputType.emailAddress)
                        //? if the isAName is true, TextCapitalization will be by words
                        ? TextCapitalization.none
                        //? if the isAName is true, TextCapitalization will be by words
                        //* else the TextCapitalization.none
                        : (widget.isAName)
                            ? TextCapitalization.words
                            : TextCapitalization.none,
                    autocorrect: false,
                    keyboardType: widget.keyboardType,
                    inputFormatters: [
                      //!isAphone or  isAName: only one of the two can be true for it to work properly
                      //When the isAPhone is true, it will only allow to enter numbers to the text field
                      if (widget.isAPhone)
                        FilteringTextInputFormatter.allow(RegExp(r'[0-9]')),
                      //When the isAPhone is true, it will only allow to enter letters to the text field
                      if (widget.isAName)
                        FilteringTextInputFormatter.allow(
                            RegExp(r'[a-z\A-Z\á\é\í\ó\ú\Á\É\Í\Ó\Ú\ ]'))
                    ],
                    decoration: InputDecoration(
                        prefixIcon: Icon(widget.icon),
                        focusedBorder: InputBorder.none,
                        border: InputBorder.none,
                        hintText: widget.placeholder),
                  )
                : TextField(
                    //the attribute enabled is used to define if the text field will be enabled or not
                    enabled: widget.enabled,
                    //The attribute textController is a TextInputType.text by default
                    controller: widget.textController,
                    // the validation to emails only workd when the attribute onChanged is null
                    onChanged:
                        (widget.keyboardType == TextInputType.emailAddress &&
                                widget.onChanged == null)
                            ? (value) {
                                final isValid = validateEmail(value);
                                setState(() {
                                  emailValid = isValid;
                                });
                              }
                            : widget.onChanged,
                    // the attribute onfocusColor is also the color to cursorColor from the TextField
                    cursorColor: widget.onfocusColor,
                    textCapitalization:
                        //the TextCapitalization.none when the keyboardType == TextInputType.emailAddress
                        (widget.keyboardType == TextInputType.emailAddress)
                            ? TextCapitalization.none
                            //? if the isAName is true, TextCapitalization will be by words
                            //* else the TextCapitalization.none
                            : (widget.isAName)
                                ? TextCapitalization.words
                                : TextCapitalization.none,
                    autocorrect: false,
                    keyboardType: widget.keyboardType,
                    inputFormatters: [
                      //!isAphone or  isAName: only one of the two can be true for it to work properly
                      //When the isAPhone is true, it will only allow to enter numbers to the text field
                      if (widget.isAPhone)
                        FilteringTextInputFormatter.allow(RegExp(r'[0-9]')),
                      //When the isAPhone is true, it will only allow to enter letters to the text field
                      if (widget.isAName)
                        FilteringTextInputFormatter.allow(
                            RegExp(r'[a-z\A-Z\á\é\í\ó\ú\Á\É\Í\Ó\Ú\ ]'))
                    ],
                    decoration: InputDecoration(
                        focusedBorder: InputBorder.none,
                        border: InputBorder.none,
                        hintText: widget.placeholder),
                  ),
      ),
    );
  }
}

// this method verifies that the email complies with the standards
bool validateEmail(String email) {
  String pattern =
      r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$';
  RegExp regExp = new RegExp(pattern);
  if (regExp.hasMatch(email)) {
    return true;
  } else {
    return false;
  }
}
